'use strict';

var _ = require('lodash'),
  mongoose = require('mongoose'),
  q = require('q'),
  Schema = mongoose.Schema,
  ObjectId = Schema.ObjectId,
  bcrypt = require('bcrypt-nodejs');

exports.createCrudController = function (Model) {
  var controller = {};

  // Get list of items
  controller.index = function (req, res) {
    Model.find(function (err, items) {
      if (err) {
        return handleError(res, err);
      }
      return res.status(200).json(items);
    });
  };

  // Get a single item
  controller.show = function (req, res) {
    Model.findById(req.params.id, function (err, item) {
      if (err) {
        return handleError(res, err);
      }
      if (!item) {
        return res.status(404).send('Not Found');
      }
      return res.json(item);
    });
  };

  // Creates a new item in the DB.
  controller.create = function (req, res) {
    Model.create(req.body, function (err, item) {
      if (err) {
        return handleError(res, err);
      }
      return res.status(201).json(item);
    });
  };

  // Updates an existing item in the DB.
  controller.update = function (req, res) {
    if (req.body._id) {
      delete req.body._id;
    }
    Model.findById(req.params.id, function (err, item) {
      if (err) {
        return handleError(res, err);
      }
      if (!item) {
        return res.status(404).send('Not Found');
      }
      var updated = _.extend(item, req.body);
      updated.save(function (err) {
        if (err) {
          return handleError(res, err);
        }
        return res.status(200).json(item);
      });
    });
  };

  // Deletes a item from the DB.
  controller.destroy = function (req, res) {
    Model.findById(req.params.id, function (err, item) {
      if (err) {
        return handleError(res, err);
      }
      if (!item) {
        return res.status(404).send('Not Found');
      }
      item.remove(function (err) {
        if (err) {
          return handleError(res, err);
        }
        return res.status(204).send('No Content');
      });
    });
  };

  // Get a item slug by name
  controller.findSlugByName = function (model, slugModel, isAdd = true) {
    var deferred = q.defer();
    try {
      model.findOne({
        name: slugModel.name,
        type: slugModel.type
      }, function (err, item) {
        if (err) {
          deferred.reject(err);
        } else if (!item || (!isAdd && (item._id.toString() == slugModel._id))) {
          deferred.resolve(true);
        } else {
          console.log(isAdd, item.id,slugModel._id)
          deferred.reject('Slug is already used by an existing object!');
        }
      });
    } catch (e) {
      deferred.reject(e);
    }
    return deferred.promise;
  }

  // Create slug
  controller.createSlug = function (model, data) {
    var deferred = q.defer();
    try {
      var id = mongoose.Types.ObjectId();
      data._id = id;
      model.create(data, function (err, item) {
        if (err) {
          deferred.reject(e);
        } else {
          deferred.resolve(item);
        }
      });
    } catch (e) {
      deferred.reject(e);
    }
    return deferred.promise;
  }

  // Update slug
  controller.updateSlug = function (model, data) {
    var deferred = q.defer();
    try {
      var id = mongoose.Types.ObjectId();
      model.update({
        _id: data._id
      }, {
          name: data.name
        }, function (err, item) {
          if (err) {
            deferred.reject(e);
          } else {
            deferred.resolve(item);
          }
        });
    } catch (e) {
      deferred.reject(e);
    }
    return deferred.promise;
  }

  function handleError(res, err) {
    return res.status(500).send(err);
  }

  return controller;
};

exports.createModel = function (name, schemaObject) {
  var schema = mongoose.Schema(_.extend(schemaObject, {
    id: ObjectId,
    tenant: {
      type: String,
      default: 'tenantA'
    },
    owner: {
      type: String
    },
    createdBy: {
      type: String
    },
    createdOn: {
      type: Date,
      default: Date.now
    },
    updatedBy: {
      type: String
    },
    updatedOn: {
      type: Date,
      default: Date.now
    }
  }));

  schema.post('save', function (doc) {
    pola.pubsub.publish(name.toLowerCase() + ':save', doc);
  });

  return mongoose.model(name, schema);
};